@@ -95,10 +95,10 @@ pref("app.update.certs.2.commonName", "aus3.mozilla.org");
 #endif
 
 // Whether or not app updates are enabled
-pref("app.update.enabled", true);               
+pref("app.update.enabled", true);
 
 // This preference turns on app.update.mode and allows automatic download and
-// install to take place. We use a separate boolean toggle for this to make     
+// install to take place. We use a separate boolean toggle for this to make
 // the UI easier to construct.
 pref("app.update.auto", true);
 
@@ -131,7 +131,7 @@ pref("app.update.url", "https://aus3.mozilla.org/update/3/%PRODUCT%/%VERSION%/%B
 // attempts fail.
 pref("app.update.url.manual", "http://www.getthunderbird.com");
 // A default value for the "More information about this update" link
-// supplied in the "An update is available" page of the update wizard. 
+// supplied in the "An update is available" page of the update wizard.
 pref("app.update.url.details", "http://www.mozilla.org/%LOCALE%/%APP%/releases/");
 // User-settable override to app.update.url for testing purposes.
 //pref("app.update.url.override", "");
@@ -142,7 +142,7 @@ pref("app.update.url.details", "http://www.mozilla.org/%LOCALE%/%APP%/releases/"
 pref("app.update.idletime", 60);
 
 // Whether or not we show a dialog box informing the user that the update was
-// successfully applied. This is off in Firefox by default since we show a 
+// successfully applied. This is off in Firefox by default since we show a
 // upgrade start page instead! Other apps may wish to show this UI, and supply
 // a whatsNewURL field in their brand.properties that contains a link to a page
 // which tells users what's new in this new update.
@@ -188,12 +188,12 @@ pref("extensions.webservice.discoverURL", "https://services.addons.mozilla.org/%
 // Blocklist preferences
 pref("extensions.blocklist.enabled", true);
 pref("extensions.blocklist.interval", 86400);
-pref("extensions.blocklist.url", "https://addons.mozilla.org/blocklist/3/%APP_ID%/%APP_VERSION%/%PRODUCT%/%BUILD_ID%/%BUILD_TARGET%/%LOCALE%/%CHANNEL%/%OS_VERSION%/%DISTRIBUTION%/%DISTRIBUTION_VERSION%/%PING_COUNT%/%TOTAL_PING_COUNT%/%DAYS_SINCE_LAST_PING%/");
+pref("extensions.blocklist.url", "https://blocklist.addons.mozilla.org/blocklist/3/%APP_ID%/%APP_VERSION%/%PRODUCT%/%BUILD_ID%/%BUILD_TARGET%/%LOCALE%/%CHANNEL%/%OS_VERSION%/%DISTRIBUTION%/%DISTRIBUTION_VERSION%/%PING_COUNT%/%TOTAL_PING_COUNT%/%DAYS_SINCE_LAST_PING%/");
 pref("extensions.blocklist.detailsURL", "https://addons.mozilla.org/%LOCALE%/%APP%/blocked/");
-pref("extensions.blocklist.itemURL", "https://addons.mozilla.org/%LOCALE%/%APP%/blocked/%blockID%");
+pref("extensions.blocklist.itemURL", "https://blocklist.addons.mozilla.org/%LOCALE%/%APP%/blocked/%blockID%");
 
-// Enables some extra Extension System Logging (can reduce performance) 
-pref("extensions.logging.enabled", false); 
+// Enables some extra Extension System Logging (can reduce performance)
+pref("extensions.logging.enabled", false);
 
 // Symmetric (can be overridden by individual extensions) update preferences.
 // e.g.
@@ -205,10 +205,10 @@ pref("extensions.logging.enabled", false);
 pref("extensions.update.enabled", true);
 pref("extensions.update.url", "https://versioncheck.addons.mozilla.org/update/VersionCheck.php?reqVersion=%REQ_VERSION%&id=%ITEM_ID%&version=%ITEM_VERSION%&maxAppVersion=%ITEM_MAXAPPVERSION%&status=%ITEM_STATUS%&appID=%APP_ID%&appVersion=%APP_VERSION%&appOS=%APP_OS%&appABI=%APP_ABI%&locale=%APP_LOCALE%&currentAppVersion=%CURRENT_APP_VERSION%&updateType=%UPDATE_TYPE%&compatMode=%COMPATIBILITY_MODE%");
 
-pref("extensions.update.interval", 86400);  // Check for updates to Extensions and 
+pref("extensions.update.interval", 86400);  // Check for updates to Extensions and
                                             // Themes every day
 
-pref("extensions.dss.enabled", false);          // Dynamic Skin Switching                                               
+pref("extensions.dss.enabled", false);          // Dynamic Skin Switching
 pref("extensions.dss.switchPending", false);    // Non-dynamic switch pending after next
 
 pref("extensions.{972ce4c6-7e08-4474-a285-3208198ce6fd}.name", "chrome://messenger/locale/messenger.properties");
@@ -293,10 +293,10 @@ pref("mail.warn_on_collapsed_thread_operation", true);
 pref("mail.warn_on_shift_delete", true);
 
 // When using commands like "next message" or "previous message", leave
-// at least this many visible rows above or below the selected message
-// in the thread pane.
-pref("mail.threadpane.padding.top", 1);
-pref("mail.threadpane.padding.bottom", 1);
+// at least this percentage of the thread pane visible above / below the
+// selected message.
+pref("mail.threadpane.padding.top_percent", 10);
+pref("mail.threadpane.padding.bottom_percent", 10);
 
 // only affects cookies from RSS articles
 // 0-Accept, 1-dontAcceptForeign, 2-dontUse
@@ -314,7 +314,7 @@ pref("mail.default_html_action", 3);
 
 /////////////////////////////////////////////////////////////////
 // End core mailnews.js pref overrides
-///////////////////////////////////////////////////////////////// 
+/////////////////////////////////////////////////////////////////
 
 /////////////////////////////////////////////////////////////////
 // Overrides for generic app behavior from the core all.js
@@ -336,9 +336,9 @@ pref("offline.download.download_messages",  0);
 #ifdef UNIX_BUT_NOT_MAC
 pref("offline.autoDetect", false);
 #else
-// Windows and Mac can automatically move the user offline or online based on 
+// Windows and Mac can automatically move the user offline or online based on
 // the network connection.
-pref("offline.autoDetect", true); 
+pref("offline.autoDetect", true);
 #endif
 
 // Expose only select protocol handlers. All others should go
@@ -384,10 +384,10 @@ pref("toolkit.telemetry.infoURL", "http://www.mozilla.org/thunderbird/legal/priv
 pref("mousewheel.withcontrolkey.action", 3);
 /////////////////////////////////////////////////////////////////
 // End core all.js pref overrides
-///////////////////////////////////////////////////////////////// 
+/////////////////////////////////////////////////////////////////
 
 /////////////////////////////////////////////////////////////////
-// Generic browser related prefs. 
+// Generic browser related prefs.
 /////////////////////////////////////////////////////////////////
 pref("browser.send_pings", false);
 pref("browser.chrome.toolbar_tips",         true);
@@ -413,7 +413,7 @@ pref("spellchecker.dictionary", "");
 pref("spellchecker.dictionaries.download.url", "https://addons.mozilla.org/%LOCALE%/%APP%/dictionaries/");
 
 // profile.force.migration can be used to bypass the migration wizard, forcing migration from a particular
-// mail application without any user intervention. Possible values are: 
+// mail application without any user intervention. Possible values are:
 // seamonkey (mozilla suite), eudora, oexpress, outlook.
 pref("profile.force.migration", "");
 
@@ -425,7 +425,7 @@ pref("alerts.totalOpenTime", 10000);
 pref("mail.phishing.detection.enabled", true);
 // If phishing detection is enabled, allow fine grained control
 // of the local, static tests
-pref("mail.phishing.detection.ipaddresses", true); 
+pref("mail.phishing.detection.ipaddresses", true);
 pref("mail.phishing.detection.mismatched_hosts", true);
 
 pref("browser.safebrowsing.enabled", false);
@@ -560,6 +560,8 @@ pref("gloda.facetview.hidetimeline", true);
 pref("mailnews.database.global.indexer.enabled", true);
 // Show gloda errors in the error console
 pref("mailnews.database.global.logging.console", true);
+// Limit the number of gloda message results
+pref("mailnews.database.global.search.msg.limit", 1000);
 
 // page to load to find good header add-ons
 pref("mailnews.migration.header_addons_url","http://live.mozillamessaging.com/%APP%/addons/search?q=header&locale=%LOCALE%&lver=%VERSION%&hver=%VERSION%&os=%OS%");
@@ -696,10 +698,10 @@ pref("browser.tabs.loadDivertedInBackground", false);
 pref("browser.chrome.site_icons", true);
 pref("browser.chrome.favicons", true);
 
-// Disable places by default as we don't want to store global history
+// Enable places by default as we want to store global history for visited links
 // Below we define reasonable defaults as copied from Firefox so that we have
-// something sensible should an extension wish to enable this.
-pref("places.history.enabled", false);
+// something sensible.
+pref("places.history.enabled", true);
 
 // With places disabled by default, the default growth increment is set to zero
 // as it would otherwise default to 10MB as the minimum space occupied by the
@@ -779,9 +781,8 @@ pref("pfs.datasource.url", "https://pfs.mozilla.org/plugins/PluginFinderService.
 php?mimetype=%PLUGIN_MIMETYPE%&appID=%APP_ID%&appVersion=%APP_VERSION%&clientOS=
 %CLIENT_OS%&chromeLocale=%CHROME_LOCALE%&appRelease=%APP_RELEASE%");
 
-// By default we show an infobar message when pages require plugins the user has
-// not installed, or are outdated.
-pref("plugins.hide_infobar_for_missing_plugin", false);
+// By default we show an infobar message when pages require plugins that are
+// outdated.
 pref("plugins.hide_infobar_for_outdated_plugin", false);
 
 #ifdef XP_MACOSX
@@ -803,13 +804,6 @@ pref("mail.taskbar.lists.enabled", true);
 pref("mail.taskbar.lists.tasks.enabled", true);
 #endif
 
-// Disable hardware accelerated layers
-pref("layers.acceleration.disabled", true);
-#ifdef XP_WIN
-// and direct2d support on Windows
-pref("gfx.direct2d.disabled", true);
-#endif
-
 // Account provisioner.
 pref("mail.provider.providerList", "https://broker-live.mozillamessaging.com/provider/list");
 pref("mail.provider.suggestFromName", "https://broker-live.mozillamessaging.com/provider/suggest");
@@ -825,17 +819,36 @@ pref("browser.search.order.1", "chrome://messenger-region/locale/region.properti
 pref("browser.search.order.2", "chrome://messenger-region/locale/region.properties");
 pref("browser.search.order.3", "chrome://messenger-region/locale/region.properties");
 
+pref("browser.search.defaultenginename.US", "data:text/plain,browser.search.defaultenginename.US=Bing");
+pref("browser.search.order.US.1", "data:text/plain,browser.search.defaultenginename.US=Bing");
+pref("browser.search.order.US.2", "data:text/plain,browser.search.defaultenginename.US=Yahoo");
+pref("browser.search.order.US.3", "data:text/plain,browser.search.defaultenginename.US=");
+
 // XXX Don't update yet, until we've verified how that affects us.
 pref("browser.search.update", false);
 
 // Check whether we need to perform engine updates every 6 hours
 pref("browser.search.update.interval", 21600);
 
+// Disable search suggestions for now
+pref("browser.search.suggest.enabled", false);
+
 // Disable remote debugging protocol logging
 pref("devtools.debugger.log", false);
 
 pref("mail.chat.enabled", true);
-pref("mail.chat.play_notification_sound", true);
+// Whether to show chat notifications or not.
+pref("mail.chat.show_desktop_notifications", true);
+// Decide how much information is to be shown in the notification.
+// 0 == Show all info (sender, chat message message preview),
+// 1 == Show sender's info only (not message preview),
+// 2 == No info (fill dummy values).
+pref("mail.chat.notification_info", 0);
+pref("mail.chat.play_sound", true);
+// 0 == default system sound, 1 == user specified wav
+pref("mail.chat.play_sound.type", 0);
+// if sound is user specified, this needs to be a file url
+pref("mail.chat.play_sound.url", "");
 // Send typing notification in private conversations
 pref("purple.conversations.im.send_typing", true);
 
@@ -848,6 +861,7 @@ pref("mail.cloud_files.learn_more_url", "https://support.mozillamessaging.com/kb
 pref("mail.ignore_thread.learn_more_url", "https://support.mozillamessaging.com/kb/ignore-threads");
 
 // Sanitize dialog window
+pref("privacy.cpd.history", true);
 pref("privacy.cpd.cookies", true);
 pref("privacy.cpd.cache", true);
 
@@ -868,3 +882,7 @@ pref("mail.main_menu.collapse_by_default", true);
 // If set to true, when saving a message to a file, use underscore
 // instead of space in the file name.
 pref("mail.save_msg_filename_underscores_for_space", false);
+
+// Disable cache v2 since migration has not been done, it is pending in bug 1021843.
+pref("browser.cache.use_new_backend",       0);
+pref("browser.cache.use_new_backend_temp",  false);
