@@ -4,6 +4,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource://gre/modules/InlineSpellChecker.jsm");
+Components.utils.import("resource://gre/modules/PlacesUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource:///modules/MailUtils.js");
 
@@ -42,6 +43,7 @@ function nsContextMenu(aXulMenu, aIsShift) {
   // Message Related Items
   this.inMessageArea = false;
   this.inThreadPane = false;
+  this.inStandaloneWindow = false;
   this.messagepaneIsBlank = false;
   this.numSelectedMessages = 0;
   this.isNewsgroup = false;
@@ -245,13 +247,16 @@ nsContextMenu.prototype = {
     if (!this.inMessageArea) {
       const messageTabSpecificItems = [
         "mailContext-openNewWindow", "threadPaneContext-openNewTab",
-        "mailContext-openConversation",
+        "mailContext-openConversation", "mailContext-openContainingFolder",
         "mailContext-archive", "mailContext-replySender",
         "mailContext-editAsNew", "mailContext-replyNewsgroup",
         "mailContext-replyAll", "mailContext-replyList",
-        "mailContext-forward", "mailContext-forwardAsAttachment",
+        "mailContext-forward", "mailContext-forwardAsMenu",
+        "mailContext-multiForwardAsAttachment",
         "mailContext-copyMessageUrl", "mailContext-moveMenu",
         "mailContext-copyMenu", "mailContext-moveToFolderAgain",
+        "mailContext-ignoreThread", "mailContext-ignoreSubthread",
+        "mailContext-watchThread",
         "mailContext-tags", "mailContext-mark", "mailContext-saveAs",
         "mailContext-printpreview", "mailContext-print", "mailContext-delete",
         "downloadSelected", "mailContext-reportPhishingURL"
@@ -273,6 +278,9 @@ nsContextMenu.prototype = {
     this.showItem("mailContext-openConversation",
                   this.numSelectedMessages == 1 && this.inThreadPane &&
                   gConversationOpener.isSelectedMessageIndexed());
+    this.showItem("mailContext-openContainingFolder",
+                  !gFolderDisplay.folderPaneVisible &&
+                  this.numSelectedMessages == 1);
 
     this.setSingleSelection("mailContext-replySender");
     this.setSingleSelection("mailContext-editAsNew");
@@ -280,8 +288,9 @@ nsContextMenu.prototype = {
     this.setSingleSelection("mailContext-replyAll");
     this.setSingleSelection("mailContext-replyList");
     this.setSingleSelection("mailContext-forward");
+    this.setSingleSelection("mailContext-forwardAsMenu");
 
-    this.showItem("mailContext-forwardAsAttachment",
+    this.showItem("mailContext-multiForwardAsAttachment",
                   this.numSelectedMessages > 1 && this.inThreadPane &&
                   !this.hideMailItems);
 
@@ -317,6 +326,23 @@ nsContextMenu.prototype = {
 
     this.showItem("mailContext-mark", msgModifyItems);
 
+    this.showItem("mailContext-ignoreThread", !this.inStandaloneWindow &&
+                                              this.numSelectedMessages >= 1 &&
+                                              !this.hideMailItems &&
+                                              !this.onPlayableMedia);
+
+    this.showItem("mailContext-ignoreSubthread", !this.inStandaloneWindow &&
+                                                 this.numSelectedMessages >= 1 &&
+                                                 !this.hideMailItems &&
+                                                 !this.onPlayableMedia);
+
+    this.showItem("mailContext-watchThread", !this.inStandaloneWindow &&
+                                             this.numSelectedMessages > 0 &&
+                                             !this.hideMailItems &&
+                                             !this.onPlayableMedia);
+
+    this.showItem("mailContext-afterWatchThread", !this.inStandaloneWindow);
+
     this.showItem("mailContext-saveAs", this.numSelectedMessages > 0 &&
                                         !this.hideMailItems &&
                                         !gMessageDisplay.isDummy &&
@@ -354,7 +380,8 @@ nsContextMenu.prototype = {
       "mailContext-sep-reply", "paneContext-afterMove",
       "mailContext-sep-afterTagAddNew", "mailContext-sep-afterTagRemoveAll",
       "mailContext-sep-afterMarkAllRead", "mailContext-sep-afterMarkFlagged",
-      "mailContext-sep-afterMarkMenu", "mailContext-sep-edit",
+      "mailContext-sep-afterMarkMenu", "mailContext-afterWatchThread",
+      "mailContext-sep-edit",
       "mailContext-sep-copy", "mailContext-sep-reportPhishing",
       "mailContext-sep-undo", "mailContext-sep-clipboard",
       "mailContext-spell-suggestions-separator", "mailContext-spell-separator",
@@ -516,11 +543,14 @@ nsContextMenu.prototype = {
       // mailTabType's list of modes, then we'll assume it is a message related
       // tab.
       this.inMessageArea = tabmail.selectedTab.mode.name in mailTabType.modes;
+      this.inStandaloneWindow = false;
     }
-    else
+    else {
       // Assume that if we haven't got a tabmail item, then we're in standalone
       // window
       this.inMessageArea = true;
+      this.inStandaloneWindow = true;
+    }
 
     if (!this.inMessageArea) {
       this.inThreadPane = false;
@@ -890,13 +920,26 @@ nsContextMenu.prototype = {
   openInBrowser: function CM_openInBrowser() {
     let uri = Services.io.newURI(this.target.ownerDocument.defaultView.
                                  top.location.href, null, null);
-
+    PlacesUtils.asyncHistory.updatePlaces({
+      uri: uri,
+      visits:  [{
+        visitDate: Date.now() * 1000,
+        transitionType: Components.interfaces.nsINavHistoryService.TRANSITION_LINK
+      }]
+    });
     Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
               .getService(Components.interfaces.nsIExternalProtocolService)
               .loadURI(uri);
   },
 
   openLinkInBrowser: function CM_openLinkInBrowser() {
+    PlacesUtils.asyncHistory.updatePlaces({
+      uri: this.linkURI,
+      visits:  [{
+        visitDate: Date.now() * 1000,
+        transitionType: Components.interfaces.nsINavHistoryService.TRANSITION_LINK
+      }]
+    });
     Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
       .getService(Components.interfaces.nsIExternalProtocolService)
       .loadURI(this.linkURI);
