@@ -78,8 +78,8 @@ var gPluginHandler = {
     browser.removeEventListener("NewPluginInstalled", gPluginHandler);
   },
 
-  getPluginUI: function (plugin, className) {
-    return plugin.ownerDocument.getAnonymousElementByAttribute(plugin, "class", className);
+  getPluginUI: function (plugin, anonId) {
+    return plugin.ownerDocument.getAnonymousElementByAttribute(plugin, "anonid", anonId);
   },
 
   get CrashSubmit() {
@@ -232,7 +232,7 @@ var gPluginHandler = {
       // The plugin binding fires this event when it is created.
       // As an untrusted event, ensure that this object actually has a binding
       // and make sure we don't handle it twice
-      let overlay = doc.getAnonymousElementByAttribute(plugin, "class", "mainBox");
+      let overlay = this.getPluginUI(plugin, "main");
       if (!overlay || overlay._bindingHandled) {
         return;
       }
@@ -252,20 +252,8 @@ var gPluginHandler = {
         break;
 
       case "PluginNotFound":
-        // For non-object plugin tags, register a click handler to install the
-        // plugin. Object tags can, and often do, deal with that themselves,
-        // so don't stomp on the page developers toes.
-        if (!(plugin instanceof HTMLObjectElement)) {
-          // We don't yet check to see if there's actually an installer available.
-          let installStatus = doc.getAnonymousElementByAttribute(plugin, "class", "installStatus");
-          installStatus.setAttribute("status", "ready");
-          let iconStatus = doc.getAnonymousElementByAttribute(plugin, "class", "icon");
-          iconStatus.setAttribute("status", "ready");
-
-          let installLink = doc.getAnonymousElementByAttribute(plugin, "anonid", "installPluginLink");
-          self.addLinkClickCallback(installLink, "installSinglePlugin", plugin);
-        }
-        /* FALLTHRU */
+        // NOP. Plugin finder service (PFS) is dead.
+        break;
 
       case "PluginBlocklisted":
       case "PluginOutdated":
@@ -276,14 +264,14 @@ var gPluginHandler = {
         break;
 
       case "PluginDisabled":
-        let manageLink = doc.getAnonymousElementByAttribute(plugin, "anonid", "managePluginsLink");
+        let manageLink = this.getPluginUI(plugin, "managePluginsLink");
         self.addLinkClickCallback(manageLink, "managePlugins");
         break;
     }
 
     // Hide the in-content UI if it's too big. The crashed plugin handler already did this.
     if (eventType != "PluginCrashed") {
-      let overlay = doc.getAnonymousElementByAttribute(plugin, "class", "mainBox");
+      let overlay = this.getPluginUI(plugin, "main");
       if (overlay != null) {
         this.setVisibility(plugin, overlay,
                            this.shouldShowOverlay(plugin, overlay));
@@ -298,34 +286,14 @@ var gPluginHandler = {
     }
   },
 
-  newPluginInstalled : function ph_newPluginInstalled(event) {
+  newPluginInstalled : function(event) {
     // browser elements are anonymous so we can't just use target.
     var browser = event.originalTarget;
 
-    // clear the plugin list, now that at least one plugin has been installed
-    browser.missingPlugins = null;
-
-    var notificationBox = getNotificationBox(browser.contentWindow);
-    var notification = notificationBox.getNotificationWithValue("missing-plugins");
-    if (notification)
-      notificationBox.removeNotification(notification);
-
     // reload the browser to make the new plugin show.
     browser.reload();
   },
 
-  // Callback for user clicking on a missing (unsupported) plugin.
-  installSinglePlugin: function ph_installSinglePlugin(plugin) {
-    var missingPluginsArray = {};
-
-    var pluginInfo = getPluginInfo(plugin);
-    missingPluginsArray[pluginInfo.mimetype] = pluginInfo;
-
-    openDialog("chrome://mozapps/content/plugins/pluginInstallerWizard.xul",
-               "PFSWindow", "chrome,centerscreen,resizable=yes",
-               {plugins: missingPluginsArray, browser: getBrowser});
-  },
-
   // Callback for user clicking on a disabled plugin
   managePlugins: function ph_managePlugins(aEvent) {
     openAddonsMgr("addons://list/plugin");
@@ -362,27 +330,20 @@ var gPluginHandler = {
                                    background: false});
   },
 
-  // event listener for missing/blocklisted/outdated/carbonFailure plugins.
-  pluginUnavailable: function ph_pluginUnavailable(plugin, eventType) {
+  // Event listener for blocklisted/outdated/carbonFailure plugins.
+  pluginUnavailable: function(plugin, eventType) {
     let Cc = Components.classes;
     let Ci = Components.interfaces;
     var tabmail = document.getElementById('tabmail');
     let browser = tabmail.getBrowserForDocument(plugin.ownerDocument
                                                 .defaultView).browser;
 
-    if (!browser.missingPlugins)
-      browser.missingPlugins = {};
-
-    var pluginInfo = getPluginInfo(plugin);
-    browser.missingPlugins[pluginInfo.mimetype] = pluginInfo;
-
     var notificationBox = getNotificationBox(browser.contentWindow);
 
     // Should only display one of these warnings per page.
-    // In order of priority, they are: outdated > missing > blocklisted
+    // In order of priority, they are: outdated > blocklisted
     let outdatedNotification = notificationBox.getNotificationWithValue("outdated-plugins");
     let blockedNotification  = notificationBox.getNotificationWithValue("blocked-plugins");
-    let missingNotification  = notificationBox.getNotificationWithValue("missing-plugins");
 
     function showBlocklistInfo() {
       var url = formatURL("extensions.blocklist.detailsURL", true);
@@ -399,17 +360,6 @@ var gPluginHandler = {
       return true;
     }
 
-    function showPluginsMissing() {
-      // get the urls of missing plugins
-      var curBrowser = tabmail.getBrowserForSelectedTab();
-      var missingPluginsArray = curBrowser.missingPlugins;
-      if (missingPluginsArray) {
-        openDialog("chrome://mozapps/content/plugins/pluginInstallerWizard.xul",
-                   "PFSWindow", "chrome,centerscreen,resizable=yes",
-                   {plugins: missingPluginsArray, browser: curBrowser});
-      }
-    }
-
 #ifdef XP_MACOSX
     function carbonFailurePluginsRestartBrowser()
     {
@@ -459,17 +409,6 @@ var gPluginHandler = {
           callback: showOutdatedPluginsInfo
         }],
       },
-      PluginNotFound: {
-        barID: "missing-plugins",
-        iconURL: "chrome://mozapps/skin/plugins/notifyPluginGeneric.png",
-        message: messengerBundle.getString("missingpluginsMessage.title"),
-        buttons: [{
-          label: messengerBundle.getString("missingpluginsMessage.button.label"),
-          accessKey: messengerBundle.getString("missingpluginsMessage.button.accesskey"),
-          popup: null,
-          callback: showPluginsMissing
-        }],
-      },
 #ifdef XP_MACOSX
       "npapi-carbon-event-model-failure": {
         barID: "carbon-failure-plugins",
@@ -485,7 +424,6 @@ var gPluginHandler = {
 #endif
     };
 
-
     // If there is already an outdated plugin notification then do nothing
     if (outdatedNotification)
       return;
@@ -509,33 +447,19 @@ var gPluginHandler = {
 #endif
 
     if (eventType == "PluginBlocklisted") {
-      if (Services.prefs.getBoolPref("plugins.hide_infobar_for_missing_plugin"))
-        return;
-
-      if (blockedNotification || missingNotification)
+      if (blockedNotification)
         return;
     }
     else if (eventType == "PluginOutdated") {
       if (Services.prefs.getBoolPref("plugins.hide_infobar_for_outdated_plugin"))
         return;
 
-      // Cancel any notification about blocklisting/missing plugins
+      // Cancel any notification about blocklisted plugins.
       if (blockedNotification)
         blockedNotification.close();
-      if (missingNotification)
-        missingNotification.close();
     }
     else if (eventType == "PluginNotFound") {
-      if (Services.prefs.getBoolPref("plugins.hide_infobar_for_missing_plugin"))
-        return;
-
-
-      if (missingNotification)
-        return;
-
-      // Cancel any notification about blocklisting plugins
-      if (blockedNotification)
-        blockedNotification.close();
+      return;
     }
 
     let notify = notifications[eventType];
@@ -571,16 +495,17 @@ var gPluginHandler = {
   // plugin in content.
   pluginInstanceCrashed: function (plugin, aEvent) {
     // Ensure the plugin and event are of the right type.
-    if (!(aEvent instanceof Components.interfaces.nsIDOMDataContainerEvent))
+    if (!(aEvent instanceof Components.interfaces.nsIDOMCustomEvent))
       return;
 
-    let submittedReport = aEvent.getData("submittedCrashReport");
+    let propBag = aEvent.detail.QueryInterface(Components.interfaces.nsIPropertyBag2);
+    let submittedReport = propBag.getPropertyAsBool("submittedCrashReport");
     let doPrompt = true; // XXX followup for .getData("doPrompt");
     let submitReports = true; // XXX followup for .getData("submitReports");
-    let pluginName = aEvent.getData("pluginName");
-    let pluginFilename = aEvent.getData("pluginFilename");
-    let pluginDumpID = aEvent.getData("pluginDumpID");
-    let browserDumpID = aEvent.getData("browserDumpID");
+    let pluginName = propBag.getPropertyAsAString("pluginName");
+    let pluginFilename = propBag.getPropertyAsAString("pluginFilename");
+    let pluginDumpID = propBag.getPropertyAsAString("pluginDumpID");
+    let browserDumpID = propBag.getPropertyAsAString("browserDumpID");
     let messengerBundle = document.getElementById("bundle_messenger");
     let tabmail = document.getElementById('tabmail');
 
@@ -597,8 +522,8 @@ var gPluginHandler = {
     // Force a layout flush so the binding is attached.
     plugin.clientTop;
     let doc = plugin.ownerDocument;
-    let overlay = doc.getAnonymousElementByAttribute(plugin, "class", "mainBox");
-    let statusDiv = doc.getAnonymousElementByAttribute(plugin, "class", "submitStatus");
+    let overlay = this.getPluginUI(plugin, "main");
+    let statusDiv = this.getPluginUI(plugin, "submitStatus");
 #ifdef MOZ_CRASHREPORTER
     let status;
 
@@ -632,7 +557,7 @@ var gPluginHandler = {
 
     statusDiv.setAttribute("status", status);
 
-    let helpIcon = doc.getAnonymousElementByAttribute(plugin, "class", "helpIcon");
+    let helpIcon = this.getPluginUI(plugin, "helpIcon");
     this.addLinkClickCallback(helpIcon, "openPluginCrashHelpPage");
 
     // If we're showing the link to manually trigger report submission, we'll
@@ -668,11 +593,11 @@ var gPluginHandler = {
     }
 #endif
 
-    let crashText = doc.getAnonymousElementByAttribute(plugin, "class", "msgCrashedText");
+    let crashText = this.getPluginUI(plugin, "crashedText");
     crashText.textContent = messageString;
     let browser = tabmail.getBrowserForSelectedTab();
 
-    let link = doc.getAnonymousElementByAttribute(plugin, "class", "reloadLink");
+    let link = this.getPluginUI(plugin, "reloadLink");
     this.addLinkClickCallback(link, "reloadPage", browser);
 
     let notificationBox = getNotificationBox(browser.contentWindow);
@@ -739,7 +664,7 @@ var gPluginHandler = {
         buttons.push(submitButton);
 #endif
 
-      let notification = notificationBox.appendNotification(messageString, "plugin-crashed",
+      notification = notificationBox.appendNotification(messageString, "plugin-crashed",
                                                             iconURL, priority, buttons);
 
       // Add the "learn more" link.
