@@ -142,7 +142,7 @@ var gAccountManager = {
       var elt = document.getElementById(aObject.id);
       elt.destroy();
       if (!elt.selected) {
-        this.accountList.removeChild(elt);
+        elt.remove();
         return;
       }
       // The currently selected element is removed,
@@ -151,7 +151,7 @@ var gAccountManager = {
       // Prevent errors if the timer is active and the account deleted
       clearTimeout(this.disableTimerID);
       this.disableTimerID = 0;
-      this.accountList.removeChild(elt);
+      elt.remove();
       var count = this.accountList.getRowCount();
       if (!count) {
         document.getElementById("accountsDesk").selectedIndex = 0;
@@ -213,12 +213,17 @@ var gAccountManager = {
   },
   addException: function am_addException() {
     let account = this.accountList.selectedItem.account;
-    if (!account.disconnected || !account.prplAccount.connectionTarget)
+    let prplAccount = account.prplAccount;
+    if (!account.disconnected || !prplAccount.connectionTarget)
       return;
 
     // Open the Gecko SSL exception dialog.
-    let params = {exceptionAdded: false, prefetchCert: true,
-                  location: account.prplAccount.connectionTarget};
+    let params = {
+      exceptionAdded: false,
+      sslStatus: prplAccount.sslStatus,
+      prefetchCert: true,
+      location: prplAccount.connectionTarget
+    };
     window.openDialog("chrome://pippki/content/exceptionDialog.xul", "",
                       "chrome,centerscreen,modal", params);
     // Reconnect the account if an exception was added.
