@@ -7,27 +7,23 @@ MOZ_APP_BASENAME=Thunderbird
 MOZ_APP_NAME=thunderbird
 MOZ_UPDATER=1
 MOZ_THUNDERBIRD=1
+MOZ_APP_STATIC_INI=1
 MOZ_CHROME_FILE_FORMAT=omni
+MOZ_DISABLE_EXPORT_JS=1
 MOZ_NO_ACTIVEX_SUPPORT=1
 MOZ_ACTIVEX_SCRIPTING_SUPPORT=
 MOZ_LDAP_XPCOM=1
 MOZ_COMPOSER=1
 if test "$OS_ARCH" = "WINNT"; then
-  if ! test "$HAVE_64BIT_OS"; then
+  if ! test "$HAVE_64BIT_BUILD"; then
     MOZ_VERIFY_MAR_SIGNATURE=1
     MOZ_MAINTENANCE_SERVICE=1
   fi
 fi
 
-# Disable Accessibility on Mac for now as unit tests fail (bug 862238).
-if test "$OS_ARCH" = "Darwin"; then
-  ACCESSIBILITY=
-fi
-
 MOZ_SAFE_BROWSING=1
 MOZ_MEDIA_NAVIGATOR=1
 MOZ_MORK=1
-MAIL_MODULE="MODULE(nsMailModule) MODULE(nsImportServiceModule)"
 
 MOZ_APP_VERSION_TXT=${_topsrcdir}/$MOZ_BUILD_APP/config/version.txt
 MOZ_APP_VERSION=`cat $MOZ_APP_VERSION_TXT`
@@ -47,3 +43,4 @@ MAR_CHANNEL_ID=thunderbird-comm-release
 if test "$OS_TARGET" = "WINNT" -o "$OS_TARGET" = "Darwin"; then
   MOZ_FOLD_LIBS=1
 fi
+MOZ_PROFILE_MIGRATOR=1
