@@ -6,20 +6,22 @@
  * Tests that the attachment reminder works properly.
  */
 
+// make SOLO_TEST=composition/test-attachment-reminder.js mozmill-one
+
 const MODULE_NAME = "test-attachment-reminder";
 
 const RELATIVE_ROOT = "../shared-modules";
 const MODULE_REQUIRES = ["folder-display-helpers",
                          "compose-helpers",
                          "window-helpers",
-                         "notificationbox-helpers"];
+                         "notificationbox-helpers",
+                         "keyboard-helpers"];
 
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource:///modules/mailServices.js");
 
 const kBoxId = "attachmentNotificationBox";
 const kNotificationId = "attachmentReminder";
-const notificationSlackTime = 1100;
 
 function setupModule(module) {
   for (let lib of MODULE_REQUIRES) {
@@ -45,6 +47,26 @@ function assert_automatic_reminder_state(aCwc, aShown) {
   return assert_notification_displayed(aCwc, kBoxId, kNotificationId, aShown);
 }
 
+/**
+ * Waits for the attachment reminder bar to change into the wished state.
+ *
+ * @param aCwc    A compose window controller.
+ * @param aShown  True for waiting for the bar to be shown, false otherwise.
+ */
+function wait_for_reminder_state(aCwc, aShown) {
+  if (aShown) {
+    // This waits up to 30 seconds for the notification to appear.
+    wait_for_notification_to_show(aCwc, kBoxId, kNotificationId);
+  } else if (check_notification_displayed(aCwc, kBoxId, kNotificationId)) {
+    // This waits up to 30 seconds for the notification to disappear.
+    wait_for_notification_to_stop(aCwc, kBoxId, kNotificationId);
+  } else {
+    // This waits 5 seconds during which the notification must not appear.
+    aCwc.sleep(5000);
+    assert_automatic_reminder_state(aCwc, false);
+  }
+}
+
 /**
  * Check whether the manual reminder is in the proper state.
  *
@@ -52,15 +74,24 @@ function assert_automatic_reminder_state(aCwc, aShown) {
  * @param aChecked  Whether the reminder should be enabled.
  */
 function assert_manual_reminder_state(aCwc, aChecked) {
-  // Check the reminder is really enabled.
-  let attachment_menu = aCwc.click_menus_in_sequence(aCwc.e("button-attachPopup"),
-                                                     [ ], true);
-  let checkedValue = aChecked ? "true" : "false";
-  assert_equals(aCwc.e("button-attachPopup_remindLaterItem").getAttribute("checked"),
-                checkedValue);
-  aCwc.close_popup_sequence(attachment_menu);
+  const remindCommand = "cmd_remindLater";
+  assert_equals(aCwc.e("button-attachPopup_remindLaterItem").getAttribute("command"),
+                remindCommand);
 
-  assert_equals(aCwc.e("cmd_remindLater").getAttribute("checked"), checkedValue);
+  let checkedValue = aChecked ? "true" : "false";
+  assert_equals(aCwc.e(remindCommand).getAttribute("checked"), checkedValue);
+}
+
+/**
+ * Returns the keywords string currently shown in the notification message.
+ *
+ * @param aCwc A compose window controller.
+ */
+function get_reminder_keywords(aCwc) {
+  assert_automatic_reminder_state(aCwc, true);
+  let nBox = aCwc.e(kBoxId);
+  let notification = nBox.getNotificationWithValue(kNotificationId);
+  return notification.querySelector("#attachmentKeywords").getAttribute("value");
 }
 
 /**
@@ -73,18 +104,16 @@ function test_attachment_reminder_appears_properly() {
   // There should be no notification yet.
   assert_automatic_reminder_state(cwc, false);
 
-  setupComposeWin(cwc, "test@example.org", "testing attachment reminder!",
+  setupComposeWin(cwc, "test@example.org", "Testing automatic reminder!",
                   "Hello! ");
 
   // Give the notification time to appear. It shouldn't.
-  cwc.sleep(notificationSlackTime);
-  if (notificationBox.getNotificationWithValue(kNotificationId))
-    throw new Error("Attachment notification shown when it shouldn't.");
+  wait_for_reminder_state(cwc, false);
 
   cwc.type(cwc.eid("content-frame"), "Seen this cool attachment?");
 
   // Give the notification time to appear. It should now.
-  wait_for_notification_to_show(cwc, kBoxId, kNotificationId);
+  wait_for_reminder_state(cwc, true);
 
   // The manual reminder should be disabled yet.
   assert_manual_reminder_state(cwc, false);
@@ -117,10 +146,14 @@ function test_attachment_reminder_dismissal() {
   assert_automatic_reminder_state(cwc, false);
 
   setupComposeWin(cwc, "test@example.org", "popping up, eh?",
-                  "Hi there, remember the attachment!");
+                  "Hi there, remember the attachment! " +
+                  "Yes, there is a file test.doc attached! " +
+                  "Do check it, test.doc is a nice attachment.");
 
   // Give the notification time to appear.
-  wait_for_notification_to_show(cwc, kBoxId, kNotificationId);
+  wait_for_reminder_state(cwc, true);
+
+  assert_equals(get_reminder_keywords(cwc), "test.doc, attachment, attached");
 
   // We didn't click the "Remind Me Later" - the alert should pop up
   // on send anyway.
@@ -137,9 +170,6 @@ function test_attachment_reminder_dismissal() {
 
   close_compose_window(cwc);
 }
-// Disabling this test on Windows due to random timeouts on our Mozmill
-// testers.
-test_attachment_reminder_dismissal.EXCLUDED_PLATFORMS = ['winnt'];
 
 /**
  * Test that the mail.compose.attachment_reminder_aggressive pref works.
@@ -156,7 +186,7 @@ function test_attachment_reminder_aggressive_pref() {
   setupComposeWin(cwc, "test@example.org", "aggressive?",
                   "Check this attachment!");
 
-  wait_for_notification_to_show(cwc, kBoxId, kNotificationId);
+  wait_for_reminder_state(cwc, true);
   click_send_and_handle_send_error(cwc);
 
   close_compose_window(cwc);
@@ -165,9 +195,6 @@ function test_attachment_reminder_aggressive_pref() {
   if (Services.prefs.prefHasUserValue(kPref))
     Services.prefs.clearUserPref(kPref);
 }
-// Disabling this test on Windows due to random timeouts on our Mozmill
-// testers.
-test_attachment_reminder_aggressive_pref.EXCLUDED_PLATFORMS = ['winnt'];
 
 /**
  * Test that clicking "No, Send Now" in the attachment reminder alert
@@ -180,7 +207,7 @@ function test_no_send_now_sends() {
                   "will the 'No, Send Now' button work?",
                   "Hello, i got your attachment!");
 
-  wait_for_notification_to_show(cwc, kBoxId, kNotificationId);
+  wait_for_reminder_state(cwc, true);
 
   // Click the send button again, this time choose "No, Send Now".
   plan_for_modal_dialog("commonDialog", click_no_send_now);
@@ -193,9 +220,6 @@ function test_no_send_now_sends() {
   // We're now back in the compose window, let's close it then.
   close_compose_window(cwc);
 }
-// Disabling this test on Windows due to random timeouts on our Mozmill
-// testers.
-test_no_send_now_sends.EXCLUDED_PLATFORMS = ['winnt'];
 
 /**
  * Click the manual reminder in the menu.
@@ -205,8 +229,10 @@ test_no_send_now_sends.EXCLUDED_PLATFORMS = ['winnt'];
  *                        of the reminder menuitem after the click.
  */
 function click_manual_reminder(aCwc, aExpectedState) {
+  wait_for_window_focused(aCwc.window);
   aCwc.click_menus_in_sequence(aCwc.e("button-attachPopup"),
                                [ {id: "button-attachPopup_remindLaterItem"} ]);
+  wait_for_window_focused(aCwc.window);
   assert_manual_reminder_state(aCwc, aExpectedState);
 }
 
@@ -231,7 +257,7 @@ function test_manual_attachment_reminder() {
   wait_for_modal_dialog("commonDialog");
 
   // Open another blank compose window.
-  let cwc = open_compose_new_mail();
+  cwc = open_compose_new_mail();
   // This one should have the reminder disabled.
   assert_manual_reminder_state(cwc, false);
   // There should be no attachment notification.
@@ -249,7 +275,7 @@ function test_manual_attachment_reminder() {
   plan_for_new_window("msgcompose");
   // ... by clicking Edit in the draft message notification bar.
   mc.click(mc.eid("msgNotificationBar", {tagName: "button", label: "Edit"}));
-  let cwc = wait_for_compose_window();
+  cwc = wait_for_compose_window();
 
   // Check the reminder enablement was preserved in the message.
   assert_manual_reminder_state(cwc, true);
@@ -266,7 +292,7 @@ function test_manual_attachment_reminder() {
 
   // Enable the manual reminder and disable it again to see if it toggles right.
   click_manual_reminder(cwc, true);
-  cwc.sleep(notificationSlackTime);
+  cwc.sleep(2000);
   click_manual_reminder(cwc, false);
 
   // Now try to send again, there should be no more alert.
@@ -295,31 +321,127 @@ function test_manual_automatic_attachment_reminder_interaction() {
                   "Expect an attachment here...");
 
   // The automatic attachment notification should pop up.
-  wait_for_notification_to_show(cwc, kBoxId, kNotificationId);
+  wait_for_reminder_state(cwc, true);
 
   // Now enable the manual reminder.
   click_manual_reminder(cwc, true);
   // The attachment notification should disappear.
-  wait_for_notification_to_stop(cwc, kBoxId, kNotificationId);
+  wait_for_reminder_state(cwc, false);
 
   // Add some more text with another keyword so the automatic notification
   // could potentially show up.
   setupComposeWin(cwc, "", "", " and find it attached!");
   // Give the notification time to appear. It shouldn't.
-  cwc.sleep(notificationSlackTime);
-  assert_automatic_reminder_state(cwc, false);
+  wait_for_reminder_state(cwc, false);
 
   // Now disable the manual reminder.
   click_manual_reminder(cwc, false);
   // Give the notification time to appear. It shouldn't.
-  cwc.sleep(notificationSlackTime);
-  assert_automatic_reminder_state(cwc, false);
+  wait_for_reminder_state(cwc, false);
 
   // Add some more text without any new keyword.
   setupComposeWin(cwc, "", "", " Did I write anything?");
   // Give the notification time to appear. It should now.
-  cwc.sleep(notificationSlackTime);
-  assert_automatic_reminder_state(cwc, true);
+  wait_for_reminder_state(cwc, true);
+
+  close_compose_window(cwc);
+}
+
+/**
+ * Assert if there is any notification in the compose window.
+ *
+ * @param aCwc         Compose Window Controller
+ * @param aValue       True if notification should exist
+ *                     False otherwise.
+ */
+function assert_no_notification(aCwc, aValue)
+{
+  let notification = aCwc.e(kBoxId).currentNotification;
+  if ((notification == null) == aValue)
+    throw new Error("Notification in wrong state");
+}
+
+/**
+ * Bug 989653
+ * Send filelink attachment should not trigger the
+ * attachment reminder.
+ */
+function test_attachment_vs_filelink_reminder() {
+  // Open a blank message compose
+  let cwc = open_compose_new_mail();
+  setupComposeWin(cwc, "test@example.invalid", "Testing Filelink notification",
+                  "There is no body. I hope you don't mind!");
+
+  // There should be no notification yet.
+  assert_no_notification(cwc, false);
+
+  // Bring up the FileLink notification.
+  let kOfferThreshold = "mail.compose.big_attachments.threshold_kb";
+  let maxSize = Services.prefs.getIntPref(kOfferThreshold, 0) * 1024;
+  add_attachment(cwc, "http://www.example.com/1", maxSize);
+
+  // The filelink attachment proposal should be up but not the attachment
+  // reminder and it should also not interfere with the sending of the message.
+  assert_notification_displayed(cwc, kBoxId, "bigAttachment", true);
+  assert_automatic_reminder_state(cwc, false);
+
+  click_send_and_handle_send_error(cwc);
+  close_window(cwc);
+}
+
+/**
+ * Bug 944643
+ * Test the attachment reminder coming up when keyword is in subject line.
+ */
+function test_attachment_reminder_in_subject() {
+  // Open a blank message compose
+  let cwc = open_compose_new_mail();
+  // This one should have the reminder disabled.
+  assert_manual_reminder_state(cwc, false);
+  // There should be no attachment notification.
+  assert_automatic_reminder_state(cwc, false);
+
+  // Add some attachment keyword in subject.
+  setupComposeWin(cwc, "test@example.invalid", "Testing attachment reminder!",
+                  "There is no keyword in this body...");
+
+  // The automatic attachment notification should pop up.
+  wait_for_reminder_state(cwc, true);
+
+  // Now clear the subject
+  delete_all_existing(cwc, cwc.eid("msgSubject"));
+
+  // Give the notification time to disappear.
+  wait_for_reminder_state(cwc, false);
+
+  close_compose_window(cwc);
+}
+
+/**
+ * Bug 944643
+ * Test the attachment reminder coming up when keyword is in subject line
+ * and also body.
+ */
+function test_attachment_reminder_in_subject_and_body() {
+  // Open a blank message compose
+  let cwc = open_compose_new_mail();
+  // This one should have the reminder disabled.
+  assert_manual_reminder_state(cwc, false);
+  // There should be no attachment notification.
+  assert_automatic_reminder_state(cwc, false);
+
+  // Add some attachment keyword in subject.
+  setupComposeWin(cwc, "test@example.invalid", "Testing attachment reminder!",
+                  "There should be an attached file in this body...");
+
+  // The automatic attachment notification should pop up.
+  wait_for_reminder_state(cwc, true);
+
+  // Now clear only the subject
+  delete_all_existing(cwc, cwc.eid("msgSubject"));
+
+  // Give the notification some time. It should not disappear.
+  wait_for_reminder_state(cwc, true);
 
   close_compose_window(cwc);
 }
@@ -372,3 +494,10 @@ function click_save_message(controller) {
                     controller.window.document.title);
   controller.window.document.documentElement.getButton('accept').doCommand();
 }
+
+function teardownModule(module) {
+  let drafts = MailServices.accounts.localFoldersServer.rootFolder
+                           .getFolderWithFlags(Ci.nsMsgFolderFlags.Drafts);
+  MailServices.accounts.localFoldersServer.rootFolder
+              .propagateDelete(drafts, true, null);
+}
