@@ -1,69 +1,35 @@
-/* ***** BEGIN LICENSE BLOCK *****
- *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Thunderbird Mail Client.
- *
- * The Initial Developer of the Original Code is
- * the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jonathan Protzenko <jonathan.protzenko@gmail.com>
- *   Mike Conley <mconley@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /**
- * Tests the get an account workflow.
+ * Tests the get an account (account provisioning) workflow.
  */
 
 var Cu = Components.utils;
 var Cc = Components.classes;
 var Ci = Components.interfaces;
 
-var MODULE_NAME = 'test-newmailaccount';
+const MODULE_NAME = 'test-newmailaccount';
 
-var RELATIVE_ROOT = '../shared-modules';
-var MODULE_REQUIRES = ['folder-display-helpers',
-                       'content-tab-helpers',
-                       'window-helpers',
-                       'newmailaccount-helpers',
-                       'keyboard-helpers',
-                       'dom-helpers'];
+const RELATIVE_ROOT = '../shared-modules';
+const MODULE_REQUIRES = ['folder-display-helpers',
+                         'content-tab-helpers',
+                         'window-helpers',
+                         'newmailaccount-helpers',
+                         'keyboard-helpers',
+                         'dom-helpers'];
 
 var controller = {};
 var mozmill = {};
 var elib = {};
-let httpd = {};
 Cu.import('resource://mozmill/modules/controller.js', controller);
 Cu.import('resource://mozmill/modules/mozmill.js', mozmill);
 Cu.import('resource://mozmill/modules/elementslib.js', elib);
 Cu.import('resource://gre/modules/Services.jsm');
+Cu.import('resource:///modules/iteratorUtils.jsm');
 Cu.import("resource:///modules/mailServices.js");
-Cu.import('resource://mozmill/stdlib/httpd.js', httpd);
+Cu.import('resource://mozmill/stdlib/httpd.js');
 
 // RELATIVE_ROOT messes with the collector, so we have to bring the path back
 // so we get the right path for the resources.
@@ -94,16 +60,11 @@ var gOldAcceptLangs = Services.prefs.getCharPref(kAcceptedLanguage);
 var gNumAccounts;
 
 function setupModule(module) {
-  let fdh = collector.getModule('folder-display-helpers');
-  fdh.installInto(module);
-  let cth = collector.getModule('content-tab-helpers');
-  cth.installInto(module);
-  let wh = collector.getModule('window-helpers');
-  wh.installInto(module);
-  let nmah = collector.getModule('newmailaccount-helpers');
-  nmah.installInto(module);
-  let kh = collector.getModule('keyboard-helpers');
-  kh.installInto(module);
+  collector.getModule('folder-display-helpers').installInto(module);
+  collector.getModule('content-tab-helpers').installInto(module);
+  collector.getModule('window-helpers').installInto(module);
+  collector.getModule('newmailaccount-helpers').installInto(module);
+  collector.getModule('keyboard-helpers').installInto(module);
   collector.getModule('dom-helpers').installInto(module);
 
   // Make sure we enable the Account Provisioner.
@@ -493,11 +454,11 @@ function subtest_html_characters_and_ampersands(w) {
   // & should have been replaced with &amp;, and the
   // greater than / less than characters with &gt; and
   // &lt; respectively.
-  assert_true(displayedName.indexOf("&amp;") != -1,
+  assert_true(displayedName.contains("&amp;"),
               "Should have eliminated ampersands");
-  assert_true(displayedName.indexOf("&gt;") != -1,
+  assert_true(displayedName.contains("&gt;"),
               "Should have eliminated greater-than signs");
-  assert_true(displayedName.indexOf("&lt;") != -1,
+  assert_true(displayedName.contains("&lt;"),
               "Should have eliminated less-than signs");
 }
 
@@ -643,7 +604,7 @@ function subtest_shows_error_on_empty_suggest_from_name(w) {
  */
 function test_throws_console_error_on_corrupt_XML() {
   // Open the provisioner - once opened, let subtest_get_an_account run.
-  get_to_order_form("corrupt@corrupt.nul");
+  get_to_order_form("corrupt@corrupt.invalid");
   let tab = mc.tabmail.currentTabInfo;
 
   // Record how many accounts we start with.
@@ -858,7 +819,7 @@ function test_can_pref_off_account_provisioner() {
   mc.click(newMailAccountMenuitem);
 
   // Ensure that the existing email account wizard opened.
-  let wizard = wait_for_new_window("mail:autoconfig");
+  wizard = wait_for_new_window("mail:autoconfig");
 
   // Make sure that the button to open the Account Provisioner dialog is
   // NOT hidden.
@@ -1032,7 +993,7 @@ function test_external_link_opening_behaviour() {
 function test_return_to_provisioner_on_error_XML() {
   const kOriginalTabNum = mc.tabmail.tabContainer.childNodes.length;
 
-  get_to_order_form("error@error.nul");
+  get_to_order_form("error@error.invalid");
 
   let tab = mc.tabmail.currentTabInfo;
   let doc = tab.browser.contentWindow.document;
@@ -1099,7 +1060,7 @@ function subtest_disabled_fields_when_searching(aController) {
   }
 
   // Set up a mock HTTP server to serve up a super slow search...
-  let server = httpd.getServer(kDefaultServerPort, '');
+  let server = new HttpServer();;
   server.registerPathHandler(kSuggestPath, slow_results);
   server.start(kDefaultServerPort);
 
@@ -1165,8 +1126,13 @@ function test_search_button_disabled_if_no_lang_support() {
   let oldLang = Services.prefs.getCharPref(kAcceptedLanguage);
   Services.prefs.setCharPref(kAcceptedLanguage, "foo");
 
-  plan_for_modal_dialog("AccountCreation",
-                        subtest_search_button_disabled_on_init);
+  plan_for_modal_dialog("AccountCreation", function(aController) {
+    wait_for_provider_list_loaded(aController);
+    // The search button should be disabled.
+    wait_for_element_enabled(aController, aController.e("searchSubmit"), false);
+    close_dialog_immediately(aController);
+  });
+
   open_provisioner_window();
   wait_for_modal_dialog("AccountCreation");
 
@@ -1175,13 +1141,18 @@ function test_search_button_disabled_if_no_lang_support() {
 
 /**
  * Subtest used by several functions that checks to make sure that the
- * search button is disabled when the Account Provisioner dialog  is opened.
+ * search button is disabled when the Account Provisioner dialog is opened,
+ * in case there's no search input yet.
  */
-function subtest_search_button_disabled_on_init(aController) {
+function subtest_search_button_enabled_state_on_init(aController) {
   wait_for_provider_list_loaded(aController);
 
-  // The search button should be disabled.
-  wait_for_element_enabled(aController, aController.e("searchSubmit"), false);
+  let enabled = !!aController.e("name").value;
+
+  // The search button should be disabled if there's not search input.
+  wait_for_element_enabled(aController, aController.e("searchSubmit"), enabled);
+
+
   close_dialog_immediately(aController);
 }
 
@@ -1230,20 +1201,17 @@ function test_search_button_disabled_if_no_query_on_init() {
   // We have to do a little bit of gymnastics to access the local storage
   // for the accountProvisioner dialog...
   let url = "chrome://content/messenger/accountProvisionerStorage/accountProvisioner";
-  let ssm = Cc["@mozilla.org/scriptsecuritymanager;1"]
-    .getService(Ci.nsIScriptSecurityManager);
-  let dsm = Cc["@mozilla.org/dom/storagemanager;1"]
-    .getService(Ci.nsIDOMStorageManager);
+  let dsm = Services.domStorageManager;
 
   let uri = Services.io.newURI(url, "", null);
-  let principal = ssm.getCodebasePrincipal(uri);
+  let principal = Services.scriptSecurityManager.getNoAppCodebasePrincipal(uri);
   let storage = dsm.getLocalStorageForPrincipal(principal, url);
 
   // Ok, got it. Now let's blank out the name.
   storage.setItem("name", "");
 
   plan_for_modal_dialog("AccountCreation",
-                        subtest_search_button_disabled_on_init);
+                        subtest_search_button_enabled_state_on_init);
   open_provisioner_window();
   wait_for_modal_dialog("AccountCreation");
 }
