@@ -73,15 +73,18 @@ nsVoidArray *dir_ServerList = nullptr;
 /*****************************************************************************
  * Functions for creating the new back end managed DIR_Server list.
  */
-class DirPrefObserver : public nsSupportsWeakReference,
-                        public nsIObserver
+class DirPrefObserver MOZ_FINAL : public nsSupportsWeakReference,
+                                  public nsIObserver
 {
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIOBSERVER
+
+private:
+  ~DirPrefObserver() {}
 };
 
-NS_IMPL_ISUPPORTS2(DirPrefObserver, nsISupportsWeakReference, nsIObserver)
+NS_IMPL_ISUPPORTS(DirPrefObserver, nsISupportsWeakReference, nsIObserver)
 
 NS_IMETHODIMP DirPrefObserver::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *aData)
 {
@@ -875,7 +878,7 @@ static void DIR_ConvertServerFileName(DIR_Server* pServer)
 {
   char* leafName = pServer->fileName;
   char* newLeafName = nullptr;
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
   /* jefft -- bug 73349 This is to allow users share same address book.
    * It only works if the user specify a full path filename.
    */
