@@ -401,7 +401,8 @@ nsMessengerOSXIntegration::ShowAlertMessage(const nsAString& aAlertTitle,
                                            NS_ConvertASCIItoUTF16(aFolderURI),
                                            this, EmptyString(),
                                            NS_LITERAL_STRING("auto"),
-                                           EmptyString(), nullptr);
+                                           EmptyString(), EmptyString(),
+                                           nullptr);
     }
 
     BounceDockIcon();
@@ -563,7 +564,7 @@ nsMessengerOSXIntegration::BadgeDockIcon()
   }
 
   id tile = [[NSApplication sharedApplication] dockTile];
-  [tile setBadgeLabel:[NSString stringWithFormat:@"%S", total.get()]];
+  [tile setBadgeLabel:[NSString stringWithFormat:@"%S", badgeString.get()]];
   return NS_OK;
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
