@@ -4,15 +4,17 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"
+#include "mozilla/mailnews/MimeHeaderParser.h"
 #include "nsMsgHdr.h"
 #include "nsMsgDatabase.h"
 #include "nsMsgUtils.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsIMsgThread.h"
 #include "nsMsgMimeCID.h"
 #include "nsIMimeConverter.h"
 
-NS_IMPL_ISUPPORTS1(nsMsgHdr, nsIMsgDBHdr)
+using namespace mozilla::mailnews;
+
+NS_IMPL_ISUPPORTS(nsMsgHdr, nsIMsgDBHdr)
 
 #define FLAGS_INITED 0x1
 #define CACHED_VALUES_INITED 0x2
@@ -28,7 +30,7 @@ nsMsgHdr::nsMsgHdr(nsMsgDatabase *db, nsIMdbRow *dbRow)
   {
     m_mdb->AddRef();
     mdbOid outOid;
-    if (dbRow && dbRow->GetOid(m_mdb->GetEnv(), &outOid) == NS_OK)
+    if (dbRow && NS_SUCCEEDED(dbRow->GetOid(m_mdb->GetEnv(), &outOid)))
     {
       m_messageKey = outOid.mOid_Id;
       m_mdb->AddHdrToUseCache((nsIMsgDBHdr *) this, m_messageKey);
@@ -61,7 +63,7 @@ nsresult nsMsgHdr::InitCachedValues()
   {
     uint32_t uint32Value;
     mdbOid outOid;
-    if (m_mdbRow->GetOid(m_mdb->GetEnv(), &outOid) == NS_OK)
+    if (NS_SUCCEEDED(m_mdbRow->GetOid(m_mdb->GetEnv(), &outOid)))
       m_messageKey = outOid.mOid_Id;
 
     err = GetUInt32Column(m_mdb->m_messageSizeColumnToken, &m_messageSize);
@@ -116,7 +118,7 @@ NS_IMETHODIMP nsMsgHdr::GetMessageKey(nsMsgKey *result)
   if (m_messageKey == nsMsgKey_None && m_mdbRow != NULL)
   {
     mdbOid outOid;
-    if (m_mdbRow->GetOid(m_mdb->GetEnv(), &outOid) == NS_OK)
+    if (NS_SUCCEEDED(m_mdbRow->GetOid(m_mdb->GetEnv(), &outOid)))
       m_messageKey = outOid.mOid_Id;
 
   }
@@ -397,101 +399,16 @@ NS_IMETHODIMP nsMsgHdr::SetRecipients(const char *recipients)
   return SetStringColumn(recipients, m_mdb->m_recipientsColumnToken);
 }
 
-nsresult nsMsgHdr::BuildRecipientsFromArray(const char *names, const char *addresses, uint32_t numAddresses, nsAutoCString& allRecipients)
-{
-  NS_ENSURE_ARG_POINTER(names);
-  NS_ENSURE_ARG_POINTER(addresses);
-  nsresult ret = NS_OK;
-  const char *curName = names;
-  const char *curAddress = addresses;
-  nsIMsgHeaderParser *headerParser = m_mdb->GetHeaderParser();
-
-  for (uint32_t i = 0; i < numAddresses; i++, curName += strlen(curName) + 1, curAddress += strlen(curAddress) + 1)
-  {
-    if (i > 0)
-      allRecipients += ", ";
-
-    if (headerParser)
-    {
-       nsCString fullAddress;
-       ret = headerParser->MakeFullAddressString(curName, curAddress,
-                                                 getter_Copies(fullAddress));
-       if (NS_SUCCEEDED(ret) && !fullAddress.IsEmpty())
-       {
-          allRecipients += fullAddress;
-          continue;
-       }
-    }
-
-        // Just in case the parser failed...
-    if (strlen(curName))
-    {
-      allRecipients += curName;
-      allRecipients += ' ';
-    }
-
-    if (strlen(curAddress))
-    {
-      allRecipients += '<';
-      allRecipients += curAddress;
-      allRecipients += '>';
-    }
-  }
-
-  return ret;
-}
-
-NS_IMETHODIMP nsMsgHdr::SetRecipientsArray(const char *names, const char *addresses, uint32_t numAddresses)
-{
-	nsresult ret;
-	nsAutoCString	allRecipients;
-
-    ret = BuildRecipientsFromArray(names, addresses, numAddresses, allRecipients);
-    if (NS_FAILED(ret))
-        return ret;
-
-	ret = SetRecipients(allRecipients.get());
-	return ret;
-}
-
 NS_IMETHODIMP nsMsgHdr::SetCcList(const char *ccList)
 {
 	return SetStringColumn(ccList, m_mdb->m_ccListColumnToken);
 }
 
-// ###should make helper routine that takes column token!
-NS_IMETHODIMP nsMsgHdr::SetCCListArray(const char *names, const char *addresses, uint32_t numAddresses)
-{
-	nsresult ret;
-	nsAutoCString	allRecipients;
-
-    ret = BuildRecipientsFromArray(names, addresses, numAddresses, allRecipients);
-    if (NS_FAILED(ret))
-        return ret;
-
-	ret = SetCcList(allRecipients.get());
-	return ret;
-}
-
 NS_IMETHODIMP nsMsgHdr::SetBccList(const char *bccList)
 {
   return SetStringColumn(bccList, m_mdb->m_bccListColumnToken);
 }
 
-NS_IMETHODIMP
-nsMsgHdr::SetBCCListArray(const char *names,
-                          const char *addresses,
-                          uint32_t numAddresses)
-{
-  nsAutoCString allRecipients;
-
-  nsresult rv = BuildRecipientsFromArray(names, addresses, numAddresses,
-                                         allRecipients);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return SetBccList(allRecipients.get());
-}
-
 NS_IMETHODIMP nsMsgHdr::SetMessageSize(uint32_t messageSize)
 {
   SetUInt32Column(messageSize, m_mdb->m_messageSizeColumnToken);
@@ -1075,10 +992,10 @@ public:
   NS_DECL_NSIUTF8STRINGENUMERATOR
 
   nsMsgPropertyEnumerator(nsMsgHdr* aHdr);
-  virtual ~nsMsgPropertyEnumerator();
   void PrefetchNext();
 
 protected:
+  virtual ~nsMsgPropertyEnumerator();
   nsCOMPtr<nsIMdbRowCellCursor> mRowCellCursor;
   nsCOMPtr<nsIMdbEnv> m_mdbEnv;
   nsCOMPtr<nsIMdbStore> m_mdbStore;
@@ -1115,7 +1032,7 @@ nsMsgPropertyEnumerator::~nsMsgPropertyEnumerator()
   mRowCellCursor = nullptr;
 }
 
-NS_IMPL_ISUPPORTS1(nsMsgPropertyEnumerator, nsIUTF8StringEnumerator)
+NS_IMPL_ISUPPORTS(nsMsgPropertyEnumerator, nsIUTF8StringEnumerator)
 
 NS_IMETHODIMP nsMsgPropertyEnumerator::GetNext(nsACString& aItem)
 {
