@@ -1,13 +1,11 @@
 /* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/*
- * Any copyright is dedicated to the Public Domain.
- * http://creativecommons.org/licenses/publicdomain/
- * Contributor: Jason Orendorff
- */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 469625;
-var summary = 'Do not assert: script->objectsOffset != 0';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
@@ -21,22 +19,22 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  function f(x) {
-    var [a, b, [c0, c1]] = [x, x, x];
-  }
 
-  expect = 'TypeError: x is null';
-  actual = 'No Error';
+// ------- Comment #92 From Jesse Ruderman
+
+  expect = 'TypeError: redeclaration of formal parameter e';
   try
   {
-    f(null);
+    eval('(function (e) { var e; const e; });');
   }
   catch(ex)
   {
     actual = ex + '';
   }
+// Without patch: "TypeError: redeclaration of var e"
+// expected new behavior // With patch:    "TypeError: redeclaration of formal parameter e:"
+
   reportCompare(expect, actual, summary);
-  
+
   exitFunc ('test');
 }
