@@ -35,6 +35,7 @@ pref("general.startup.addressbook",         false);
 
 pref("general.open_location.last_url",      "");
 pref("general.open_location.last_window_choice", 0);
+pref("browser.urlbar.historyEnabled",       true);
 
 pref("general.smoothScroll", false);
 pref("general.autoScroll", true);
@@ -102,6 +103,7 @@ pref("browser.search.log", false);
 // Ordering of Search Engines in the Engine list.
 pref("browser.search.order.1", "chrome://communicator-region/locale/region.properties");
 pref("browser.search.order.2", "chrome://communicator-region/locale/region.properties");
+pref("browser.search.order.3", "chrome://communicator-region/locale/region.properties");
 
 // Search (side)bar results always open in a new tab.
 pref("browser.search.openintab", false);
@@ -329,8 +331,21 @@ pref("browser.offline-apps.notify", true);
 
 pref("browser.formfill.expire_days",        180);
 
+// Handle mail/news URLs internally by default...
+pref("network.protocol-handler.external.mailto", false); // for mail
+pref("network.protocol-handler.external.news", false);   // for news
+pref("network.protocol-handler.external.snews", false);  // for secure news
+pref("network.protocol-handler.external.nntp", false);   // also news
+
+// ...but still show the dialog at least the first time if switched to external
+pref("network.protocol-handler.warn-external.mailto", true);
+pref("network.protocol-handler.warn-external.news", true);
+pref("network.protocol-handler.warn-external.snews", true);
+pref("network.protocol-handler.warn-external.nntp", true);
+
 // bug 1005566 - Disable seer until properly supported
-pref("network.seer.enabled", false);
+// bug 1021370 - Rename Seer to Predictor
+pref("network.predictor.enabled", false);
 
 pref("mail.biff.show_new_alert",     true);
 
@@ -597,9 +612,9 @@ pref("extensions.blocklist.interval", 86400);
 // Controls what level the blocklist switches from warning about items to forcibly
 // blocking them.
 pref("extensions.blocklist.level", 2);
-pref("extensions.blocklist.url", "https://addons.mozilla.org/blocklist/3/%APP_ID%/%APP_VERSION%/%PRODUCT%/%BUILD_ID%/%BUILD_TARGET%/%LOCALE%/%CHANNEL%/%OS_VERSION%/%DISTRIBUTION%/%DISTRIBUTION_VERSION%/%PING_COUNT%/%TOTAL_PING_COUNT%/%DAYS_SINCE_LAST_PING%/");
+pref("extensions.blocklist.url", "https://blocklist.addons.mozilla.org/blocklist/3/%APP_ID%/%APP_VERSION%/%PRODUCT%/%BUILD_ID%/%BUILD_TARGET%/%LOCALE%/%CHANNEL%/%OS_VERSION%/%DISTRIBUTION%/%DISTRIBUTION_VERSION%/%PING_COUNT%/%TOTAL_PING_COUNT%/%DAYS_SINCE_LAST_PING%/");
 pref("extensions.blocklist.detailsURL", "https://www.mozilla.com/%LOCALE%/blocklist/");
-pref("extensions.blocklist.itemURL", "https://addons.mozilla.org/%LOCALE%/%APP%/blocked/%blockID%");
+pref("extensions.blocklist.itemURL", "https://blocklist.addons.mozilla.org/%LOCALE%/%APP%/blocked/%blockID%");
 
 // Update preferences for installed Extensions and Themes.
 // Symmetric (can be overridden by individual extensions),
@@ -790,14 +805,17 @@ pref("security.alternate_certificate_error_page", "certerror");
 pref("security.warn_entering_secure", false);
 pref("security.warn_leaving_secure", false);
 pref("security.warn_submit_insecure", false);
-pref("security.warn_viewing_mixed", true);
+pref("security.warn_viewing_mixed", false);
 pref("security.warn_mixed_active_content", true);
-pref("security.warn_mixed_display_content", false);
+pref("security.warn_mixed_display_content", true);
 // Block insecure active content on https pages
 pref("security.mixed_content.block_active_content", true);
 // Turn on the CSP 1.0 parser for Content Security Policy headers
 pref("security.csp.speccompliant", true);
 
+// REMOVE once bug 903439 is fixed (no geolocation API key at this time)
+pref("geo.enabled", false);
+
 // FAQ URLs
 pref("browser.geolocation.warning.infoURL", "http://www.seamonkey-project.org/doc/2.0/geolocation");
 
@@ -835,6 +853,7 @@ pref("services.sync.prefs.sync.browser.download.manager.scanWhenDone", true);
 pref("services.sync.prefs.sync.browser.formfill.enable", true);
 pref("services.sync.prefs.sync.browser.link.open_external", true);
 pref("services.sync.prefs.sync.browser.link.open_newwindow", true);
+pref("services.sync.prefs.sync.browser.offline-apps.notify", true);
 pref("services.sync.prefs.sync.browser.safebrowsing.enabled", true);
 pref("services.sync.prefs.sync.browser.safebrowsing.malware.enabled", true);
 pref("services.sync.prefs.sync.browser.search.update", true);
@@ -895,12 +914,11 @@ pref("services.sync.prefs.sync.mailnews.offline_sync_news", true);
 pref("services.sync.prefs.sync.mailnews.offline_sync_send_unsent", true);
 pref("services.sync.prefs.sync.mailnews.offline_sync_work_offline", true);
 pref("services.sync.prefs.sync.mailnews.remember_selected_message", true);
-pref("services.sync.prefs.sync.mailnews.reply_header_authorwrote", true);
-pref("services.sync.prefs.sync.mailnews.reply_header_colon", true);
+pref("services.sync.prefs.sync.mailnews.reply_header_authorwrotesingle", true);
+pref("services.sync.prefs.sync.mailnews.reply_header_ondateauthorwrote", true);
+pref("services.sync.prefs.sync.mailnews.reply_header_authorwroteondate", true);
 pref("services.sync.prefs.sync.mailnews.reply_header_locale", true);
-pref("services.sync.prefs.sync.mailnews.reply_header_ondate", true);
 pref("services.sync.prefs.sync.mailnews.reply_header_originalmessage", true);
-pref("services.sync.prefs.sync.mailnews.reply_header_separator", true);
 pref("services.sync.prefs.sync.mailnews.reply_header_type", true);
 pref("services.sync.prefs.sync.mailnews.scroll_to_new_message", true);
 pref("services.sync.prefs.sync.mailnews.sendInBackground", true);
@@ -915,6 +933,7 @@ pref("services.sync.prefs.sync.mailnews.view_default_charset", true);
 pref("services.sync.prefs.sync.mailnews.wraplength", true);
 pref("services.sync.prefs.sync.network.cookie.cookieBehavior", true);
 pref("services.sync.prefs.sync.network.cookie.lifetimePolicy", true);
+pref("services.sync.prefs.sync.offline-apps.allow_by_default", true);
 pref("services.sync.prefs.sync.permissions.default.image", true);
 pref("services.sync.prefs.sync.privacy.donottrackheader.enabled", true);
 pref("services.sync.prefs.sync.privacy.donottrackheader.value", true);
