@@ -186,7 +186,7 @@ var StarUI = {
 
     gEditItemOverlay.initPanel(this._itemId,
                                { hiddenRows: ["description", "location",
-                                              "loadInSidebar", "keyword"] });
+                                              "keyword"] });
   },
 
   panelShown:
@@ -285,7 +285,7 @@ var PlacesCommandHook = {
       PlacesUtils.transactionManager.doTransaction(txn);
       // Set the character-set
       if (charset)
-        PlacesUtils.history.setCharsetForURI(uri, charset);
+        PlacesUtils.setCharsetForURI(uri, charset);
       itemId = PlacesUtils.getMostRecentBookmarkForURI(uri);
     }
 
@@ -329,7 +329,7 @@ var PlacesCommandHook = {
 
     var insertPoint = new InsertionPoint(parent,
                                          PlacesUtils.bookmarks.DEFAULT_INDEX);
-    var hiddenRows = ["loadInSidebar"];
+    var hiddenRows = [];
     PlacesUIUtils.showAddBookmarkUI(uri, title, description, insertPoint, true,
                                     null, null, null, null, hiddenRows);
   },
@@ -725,7 +725,7 @@ var PlacesStarButton = {
       delete this._pendingStmt;
     }
 
-    this._pendingStmt = PlacesUtils.asyncGetBookmarkIds(this._uri, function (aItemIds, aURI) {
+    this._pendingStmt = PlacesUtils.asyncGetBookmarkIds(this._uri, (aItemIds, aURI) => {
       // Safety check that the bookmarked URI equals the tracked one.
       if (!aURI.equals(this._uri)) {
         Components.utils.reportError("PlacesStarButton did not receive current URI");
@@ -751,7 +751,7 @@ var PlacesStarButton = {
       }
 
       delete this._pendingStmt;
-    }, this);
+    });
   },
 
   _updateStateInternal: function PSB__updateStateInternal()
