@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla History System
- *
- * The Initial Developer of the Original Code is
- * Google Inc.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Brett Wilson <brettw@gmail.com> (Original author)
- *   Asaf Romano <mano@mozilla.com> (JavaScript version)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 function PlacesTreeView() {
   this._tree = null;
@@ -46,20 +12,6 @@ function PlacesTreeView() {
 }
 
 PlacesTreeView.prototype = {
-  _makeAtom: function PTV__makeAtom(aString) {
-    return Components.classes["@mozilla.org/atom-service;1"]
-                     .getService(Components.interfaces.nsIAtomService)
-                     .getAtom(aString);
-  },
-
-  _atoms: [],
-  _getAtomFor: function PTV__getAtomFor(aName) {
-    if (!this._atoms[aName])
-      this._atoms[aName] = this._makeAtom(aName);
-
-    return this._atoms[aName];
-  },
-
   __dateService: null,
   get _dateService() {
     if (!this.__dateService) {
@@ -122,16 +74,13 @@ PlacesTreeView.prototype = {
    * @return true if aContainer is a plain container, false otherwise.
    */
   _isPlainContainer: function PTV__isPlainContainer(aContainer) {
-    if (aContainer._plainContainer !== undefined)
-      return aContainer._plainContainer;
-
     // All history containers are query containers, but we need to QI
     aContainer.QueryInterface(Components.interfaces.nsINavHistoryQueryResultNode);
 
     // Of all history containers, only URI containers are flat, and they don't
     // contain folders, no need to check that either.
-    return aContainer._plainContainer = (aContainer.queryOptions.resultType ==
-               Components.interfaces.nsINavHistoryQueryOptions.RESULTS_AS_URI)
+    return aContainer.queryOptions.resultType ==
+               Components.interfaces.nsINavHistoryQueryOptions.RESULTS_AS_URI;
   },
 
   /**
@@ -161,12 +110,21 @@ PlacesTreeView.prototype = {
   _getRowForNode:
   function PTV__getRowForNode(aNode, aForceBuild, aParentRow, aNodeIndex) {
     if (aNode == this._rootNode)
-      throw "The root node is never visible";
+      throw new Error("The root node is never visible");
 
-    let ancestors = PlacesUtils.nodeAncestors(aNode);
-    for (let ancestor in ancestors) {
+    // A node is removed form the view either if it has no parent or if its
+    // root-ancestor is not the root node (in which case that's the node
+    // for which nodeRemoved was called).
+    let ancestors = [x for each (x in PlacesUtils.nodeAncestors(aNode))];
+    if (ancestors.length == 0 ||
+        ancestors[ancestors.length - 1] != this._rootNode) {
+      throw new Error("Removed node passed to _getRowForNode");
+    }
+
+    // Ensure that the entire chain is open, otherwise that node is invisible.
+    for (let ancestor of ancestors) {
       if (!ancestor.containerOpen)
-        throw "Invisible node passed to _getRowForNode";
+        throw new Error("Invisible node passed to _getRowForNode");
     }
 
     // Non-plain containers are initially built with their contents.
@@ -286,8 +244,6 @@ PlacesTreeView.prototype = {
     if (this._isPlainContainer(aContainer))
       return cc;
 
-    const openLiteral = PlacesUIUtils.RDF.GetResource("http://home.netscape.com/NC-rdf#open");
-    const trueLiteral = PlacesUIUtils.RDF.GetLiteral("true");
     var sortingMode = this._result.sortingMode;
 
     var rowsInserted = 0;
@@ -302,10 +258,9 @@ PlacesTreeView.prototype = {
 
       // recursively do containers
       if (curChild instanceof Components.interfaces.nsINavHistoryContainerResultNode) {
-        var resource = this._getResourceForNode(curChild);
-        var isopen = resource != null &&
-                     PlacesUIUtils.localStore.HasAssertion(resource, openLiteral,
-                                                           trueLiteral, true);
+        NS_ASSERT(curChild.uri, "if there is no uri, we can't persist the open state");
+        var isopen = curChild.uri &&
+                     PlacesUIUtils.xulStore.hasValue(location, curChild.uri, "open");
         if (isopen != curChild.containerOpen)
           aToOpen.push(curChild);
         else if (curChild.containerOpen && curChild.childCount > 0)
@@ -855,8 +810,16 @@ PlacesTreeView.prototype = {
       this._rootNode.containerOpen = false;
     }
 
-    this._result = val;
-    this._rootNode = val ? val.root : null;
+    if (val) {
+      this._result = val;
+      this._rootNode = val.root;
+      this._cellProperties = new Map();
+    }
+    else if (this._result) {
+      delete this._result;
+      delete this._rootNode;
+      delete this._cellProperties;
+    }
 
     // If the tree is not set yet, setTree will call finishInit.
     if (this._tree && val)
@@ -882,13 +845,6 @@ PlacesTreeView.prototype = {
     return Components.interfaces.nsINavHistoryResultTreeViewer.INDEX_INVISIBLE;
   },
 
-  _getResourceForNode: function PTV_getResourceForNode(aNode)
-  {
-    var uri = aNode.uri;
-    NS_ASSERT(uri, "if there is no uri, we can't persist the open state");
-    return uri ? PlacesUIUtils.RDF.GetResource(uri) : null;
-  },
-
   // nsITreeView
   get rowCount() {
     return this._rows.length;
@@ -902,30 +858,30 @@ PlacesTreeView.prototype = {
     return this._selection = val;
   },
 
-  getRowProperties: function PTV_getRowProperties(aRow, aProperties) { },
+  getRowProperties: function PTV_getRowProperties(aRow) { return ""; },
 
-  getCellProperties: function PTV_getCellProperties(aRow, aColumn, aProperties) {
+  getCellProperties: function PTV_getCellProperties(aRow, aColumn) {
     if (aColumn.id != "Name")
-      return;
+      return "";
 
-    var node = this._getNodeForRow(aRow);
-    if (!node._cellProperties) {
-      let properties = new Array();
+    let node = this._getNodeForRow(aRow);
+    let properties = this._cellProperties.get(node);
+    if (!properties) {
+      properties = "Name";
       if (node.type == Components.interfaces.nsINavHistoryResultNode.RESULT_TYPE_QUERY) {
-        properties.push(this._getAtomFor("query"));
+        properties += " query";
         if (PlacesUtils.nodeIsDay(node))
-          properties.push(this._getAtomFor("dayContainer"));
+          properties += " dayContainer";
         else if (PlacesUtils.nodeIsHost(node))
-          properties.push(this._getAtomFor("hostContainer"));
+          properties += " hostContainer";
       }
 
-      node._cellProperties = properties;
+      this._cellProperties.set(node, properties);
     }
-    for (let i = 0; i < node._cellProperties.length; i++)
-      aProperties.AppendElement(node._cellProperties[i]);
+    return properties;
   },
 
-  getColumnProperties: function(aColumn, aProperties) { },
+  getColumnProperties: function(aColumn) { return ""; },
 
   isContainer: function PTV_isContainer(aRow) {
     // Only leaf nodes aren't listed in the rows array.
@@ -1072,16 +1028,13 @@ PlacesTreeView.prototype = {
       throw Components.results.NS_ERROR_UNEXPECTED;
 
     var node = this._rows[aRow];
+    NS_ASSERT(node.uri, "if there is no uri, we can't persist the open state");
 
-    var resource = this._getResourceForNode(node);
-    if (resource) {
-      const openLiteral = PlacesUIUtils.RDF.GetResource("http://home.netscape.com/NC-rdf#open");
-      const trueLiteral = PlacesUIUtils.RDF.GetLiteral("true");
-
+    if (node.uri) {
       if (node.containerOpen)
-        PlacesUIUtils.localStore.Unassert(resource, openLiteral, trueLiteral);
+        PlacesUIUtils.xulStore.removeValue(location, node.uri, "open");
       else
-        PlacesUIUtils.localStore.Assert(resource, openLiteral, trueLiteral, true);
+        PlacesUIUtils.xulStore.setValue(location, node.uri, "open", "true");
     }
 
     // 474287 Places enforces title sorting for groups, which we don't want.
