@@ -8,6 +8,7 @@ Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/AddonManager.jsm");
 Components.utils.import("resource://gre/modules/LoginManagerContent.jsm");
+Components.utils.import("resource://gre/modules/LoginManagerParent.jsm");
 Components.utils.import("resource:///modules/Sanitizer.jsm");
 Components.utils.import("resource:///modules/mailnewsMigrator.js");
 
@@ -32,6 +33,12 @@ XPCOMUtils.defineLazyModuleGetter(this, "PlacesBackups",
 XPCOMUtils.defineLazyModuleGetter(this, "BookmarkHTMLUtils",
                                   "resource://gre/modules/BookmarkHTMLUtils.jsm");
 
+XPCOMUtils.defineLazyModuleGetter(this, "BookmarkJSONUtils",
+                                  "resource://gre/modules/BookmarkJSONUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetter(this, "Task",
+                                  "resource://gre/modules/Task.jsm");
+
 XPCOMUtils.defineLazyModuleGetter(this, "DebuggerServer",
                                   "resource://gre/modules/devtools/dbg-server.jsm");
 
@@ -45,6 +52,7 @@ const BOOKMARKS_BACKUP_MAX_BACKUPS = 10;
 // Devtools Preferences
 const DEBUGGER_REMOTE_ENABLED = "devtools.debugger.remote-enabled";
 const DEBUGGER_REMOTE_PORT = "devtools.debugger.remote-port";
+const DEBUGGER_FORCE_LOCAL = "devtools.debugger.force-local";
 
 // Constructor
 
@@ -125,6 +133,7 @@ SuiteGlue.prototype = {
               this.dbgStop();
             break;
           case DEBUGGER_REMOTE_PORT:
+          case DEBUGGER_FORCE_LOCAL:
             /**
              * If the server is not on, port changes have nothing to affect.
              * The new value will be picked up if the server is started.
@@ -145,6 +154,8 @@ SuiteGlue.prototype = {
         this._promptForMasterPassword();
         this._checkForNewAddons();
         Services.search.init();
+        LoginManagerParent.init();
+        Components.utils.import("resource://gre/modules/Webapps.jsm");
         break;
       case "sessionstore-windows-restored":
         this._onBrowserStartup(subject);
@@ -233,6 +244,13 @@ SuiteGlue.prototype = {
         else
           ss.currentEngine = ss.defaultEngine;
         break;
+      case "timer-callback":
+        // Load the Login Manager data from disk off the main thread, some time
+        // after startup.  If the data is required before the timeout, for example
+        // because a restored page contains a password field, it will be loaded on
+        // the main thread, and this initialization request will be ignored.
+        Services.logins;
+        break;
     }
   },
 
@@ -330,6 +348,10 @@ SuiteGlue.prototype = {
     }
 
     this._setUpUserAgentOverrides();
+
+    var timer = Components.classes["@mozilla.org/timer;1"]
+                          .createInstance(Components.interfaces.nsITimer);
+    timer.init(this, 3000, timer.TYPE_ONE_SHOT);
   },
 
   _setUpUserAgentOverrides: function ()
@@ -701,11 +723,11 @@ SuiteGlue.prototype = {
    *   Set to true by safe-mode dialog to indicate we must restore default
    *   bookmarks.
    */
-  _initPlaces: function(aInitialMigrationPerformed) {
+  _initPlaces: Task.async(function(aInitialMigrationPerformed) {
     // We must instantiate the history service since it will tell us if we
     // need to import or restore bookmarks due to first-run, corruption or
     // forced migration (due to a major schema change).
-    var bookmarksBackupFile = PlacesBackups.getMostRecent("json");
+    var bookmarksBackupFile = yield PlacesBackups.getMostRecentBackup();
 
     // If the database is corrupt or has been newly created we should
     // import bookmarks. Same if we don't have any JSON backups, which
@@ -744,7 +766,7 @@ SuiteGlue.prototype = {
       // Get latest JSON backup.
       if (bookmarksBackupFile) {
         // Restore from JSON backup.
-        PlacesUtils.restoreBookmarksFromJSONFile(bookmarksBackupFile);
+        yield BookmarkJSONUtils.importFromFile(bookmarksBackupFile, true);
         importBookmarks = false;
       }
       else if (dbStatus == PlacesUtils.history.DATABASE_STATUS_OK) {
@@ -835,7 +857,7 @@ SuiteGlue.prototype = {
       this._idleService.addIdleObserver(this, BOOKMARKS_BACKUP_IDLE_TIME);
       this._isIdleObserver = true;
     }
-  },
+  }),
 
   /**
    * Places shut-down tasks
@@ -862,7 +884,7 @@ SuiteGlue.prototype = {
         // potential hangs (bug 518683).  The asynchronous shutdown operations
         // will then be handled by a shutdown service (bug 435058).
         var shutdownComplete = false;
-        BookmarkHTMLUtils.exportToFile(FileUtils.getFile("BMarks", [])).then(
+        BookmarkHTMLUtils.exportToFile(Services.dirsvc.get("BMarks", Components.interfaces.nsILocalFile)).then(
           function onSuccess() {
             shutdownComplete = true;
           },
@@ -951,13 +973,15 @@ SuiteGlue.prototype = {
       DebuggerServer.init();
       DebuggerServer.addBrowserActors();
     }
-    DebuggerServer.openListener(port);
+    try {
+      DebuggerServer.openListener(port);
+    } catch(e) {}
   },
 
   dbgStop: function()
   {
     if (DebuggerServer.initialized)
-      DebuggerServer.closeListener();
+      DebuggerServer.closeAllListeners();
   },
 
   dbgRestart: function()
@@ -1166,12 +1190,17 @@ ContentPermissionPrompt.prototype = {
 
   prompt: function(aRequest)
   {
+    // Only allow exactly one permission type here.
+    if (aRequest.types.length != 1)
+      return;
+
     const kFeatureKeys = { "geolocation" : "geo",
                            "desktop-notification" : "desktop-notification",
                          };
 
     // Make sure that we support the request.
-    if (!(aRequest.type in kFeatureKeys))
+    var type = aRequest.types.queryElementAt(0, Components.interfaces.nsIContentPermissionType).type;
+    if (!(type in kFeatureKeys))
       return;
 
     var path, host;
@@ -1186,7 +1215,7 @@ ContentPermissionPrompt.prototype = {
     else
       return;
 
-    var perm = kFeatureKeys[aRequest.type];
+    var perm = kFeatureKeys[type];
     switch (Services.perms.testExactPermissionFromPrincipal(requestingPrincipal, perm)) {
       case Services.perms.ALLOW_ACTION:
         aRequest.allow();
@@ -1219,7 +1248,7 @@ ContentPermissionPrompt.prototype = {
                      .chromeEventHandler.parentNode;
 
     // Show the prompt.
-    switch (aRequest.type) {
+    switch (type) {
       case "geolocation":
         nb.showGeolocationPrompt(path, host, allowCallback, cancelCallback);
         break;
