@@ -346,7 +346,7 @@ function onViewToolbarsPopupShowing(aEvent, aInsertPoint)
   // Empty the menu
   var deadItems = popup.getElementsByAttribute("toolbarid", "*");
   for (let i = deadItems.length - 1; i >= 0; --i)
-    popup.removeChild(deadItems[i]);
+    deadItems[i].remove();
 
   // Thunderbird/Lightning function signature is:
   // onViewToolbarsPopupShowing(aEvent, toolboxIds, aInsertPoint)
@@ -1118,7 +1118,16 @@ function BrowserOnCommand(event)
   // Exception" or "Get Me Out Of Here" button
   if (docURI.startsWith("about:certerror?")) {
     if (buttonID == "exceptionDialogButton") {
-      var params = { exceptionAdded : false };
+      const Ci = Components.interfaces;
+      let docshell = ownerDoc.defaultView
+                             .QueryInterface(Ci.nsIInterfaceRequestor)
+                             .getInterface(Ci.nsIWebNavigation)
+                             .QueryInterface(Ci.nsIDocShell);
+      let securityInfo = docshell.failedChannel.securityInfo;
+      let sslStatus = securityInfo.QueryInterface(Ci.nsISSLStatusProvider)
+                                  .SSLStatus;
+
+      let params = { exceptionAdded : false, sslStatus : sslStatus };
 
       switch (GetIntPref("browser.ssl_override_behavior", 2)) {
         case 2 : // Pre-fetch & pre-populate.
@@ -1229,8 +1238,8 @@ function popupNotificationMenuShowing(event)
 
 function RemovePopupsItems(parent)
 {
-  while (parent.lastChild && ("popup" in parent.lastChild))
-    parent.removeChild(parent.lastChild);
+  while (parent.lastChild && parent.lastChild.hasAttribute("popupReportIndex"))
+    parent.lastChild.remove();
 }
 
 function createShowPopupsMenu(parent, browser)
@@ -1238,32 +1247,37 @@ function createShowPopupsMenu(parent, browser)
   if (!browser)
     return false;
 
-  var popups = browser.pageReport;
+  var popups = browser.blockedPopups;
 
   if (!popups)
     return false;
 
+  parent.browser = browser;
+
   for (var i = 0; i < popups.length; i++) {
-    var popup = popups[i];
-    var menuitem = document.createElement("menuitem");
-    var str = gUtilityBundle.getFormattedString("popupMenuShow",
-                                                [popup.popupWindowURI.spec]);
-    menuitem.setAttribute("label", str);
-    menuitem.popup = popup;
+    // popupWindowURI will be null if a file input was blocked.
+    var URI = popups[i].popupWindowURI;
+    if (!URI)
+      continue;
+
+    var str = gUtilityBundle.getFormattedString("popupMenuShow", [URI.spec]);
+    // Check for duplicates and reuse the old menuitem.
+    var menuitem = parent.getElementsByAttribute("label", str).item(0);
+    if (!menuitem) {
+      menuitem = document.createElement("menuitem");
+      menuitem.setAttribute("label", str);
+    }
+    menuitem.setAttribute("popupReportIndex", i);
     parent.appendChild(menuitem);
   }
 
-  return true;
+  return parent.getElementsByAttribute("popupReportIndex", "*").item(0) != null;
 }
 
 function popupBlockerMenuCommand(target)
 {
-  if (!("popup" in target))
-    return;
-  var popup = target.popup;
-  var reqWin = popup.requestingWindow;
-  if (reqWin.document == popup.requestingDocument)
-    reqWin.open(popup.popupWindowURI.spec, popup.popupWindowName, popup.popupWindowFeatures);
+  if (target.hasAttribute("popupReportIndex"))
+    target.parentNode.browser.unblockPopup(target.getAttribute("popupReportIndex"));
 }
 
 function disablePopupBlockerNotifications()
