@@ -13,7 +13,7 @@ MOZ_COMPOSER=1
 MOZ_SUITE=1
 MOZ_BRANDING_DIRECTORY=suite/branding/nightly
 MOZ_OFFICIAL_BRANDING_DIRECTORY=suite/branding/nightly
-MOZ_EXTENSIONS_DEFAULT=" venkman inspector irc gnomevfs"
+MOZ_EXTENSIONS_DEFAULT=" inspector irc gio"
 MOZ_UPDATER=1
 # This should usually be the same as the value MAR_CHANNEL_ID.
 # If more than one ID is needed, then you should use a comma separated list
@@ -25,12 +25,13 @@ MOZ_HELP_VIEWER=1
 MOZ_MEDIA_NAVIGATOR=1
 MOZ_MORK=1
 MOZ_STATIC_BUILD_UNSUPPORTED=1
-MAIL_COMPONENT="mail import"
-MAIL_MODULE="MODULE(nsMailModule) MODULE(nsImportServiceModule)"
 MOZ_SERVICES_COMMON=1
 MOZ_SERVICES_CRYPTO=1
+MOZ_SERVICES_FXACCOUNTS=1
 MOZ_SERVICES_SYNC=1
 MOZ_URL_CLASSIFIER=1
+MOZ_WEBGL_CONFORMANT=1
+MOZ_WEBM_ENCODER=1
 
 MOZ_APP_VERSION_TXT=${_topsrcdir}/$MOZ_BUILD_APP/config/version.txt
 MOZ_APP_VERSION=`cat $MOZ_APP_VERSION_TXT`
@@ -40,8 +41,9 @@ MOZ_UA_BUILDID=20100101
 
 MOZ_APP_ID={92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}
 MOZ_PROFILE_MIGRATOR=1
-MOZ_EXTENSION_MANAGER=1
 MOZ_APP_STATIC_INI=1
 if test "$OS_TARGET" = "WINNT" -o "$OS_TARGET" = "Darwin"; then
   MOZ_FOLD_LIBS=1
 fi
+# Enable generational GC on desktop.
+JSGC_GENERATIONAL=1
