@@ -634,6 +634,7 @@ function updateEditItems()
   goUpdateCommand("cmd_renameAttachment");
   goUpdateCommand("cmd_selectAll");
   goUpdateCommand("cmd_openAttachment");
+  goUpdateCommand("cmd_findReplace");
   goUpdateCommand("cmd_find");
   goUpdateCommand("cmd_findNext");
   goUpdateCommand("cmd_findPrev");
@@ -1199,7 +1200,7 @@ function ComposeUnload()
 function ComposeSetCharacterSet(aEvent)
 {
   if (gMsgCompose) {
-    gMsgCompose.SetDocumentCharset(aEvent.target.id.slice("charset.".length));
+    gMsgCompose.SetDocumentCharset(aEvent.target.getAttribute("charset"));
     gCharsetTitle = null;
     SetComposeWindowTitle();
   }
@@ -1286,7 +1287,7 @@ function GenericSendMessage( msgType )
           window.cancelSendMessage = false;
           try {
             window.openDialog("chrome://editor/content/EdSpellCheck.xul", "_blank",
-                    "chrome,close,titlebar,modal", true, true, false);
+                    "dialog,close,titlebar,modal,resizable", true, true, false);
           }
           catch(ex){}
           if(window.cancelSendMessage)
@@ -1789,7 +1790,7 @@ function InitLanguageMenu()
 
   // Remove any languages from the list.
   while (languageMenuList.hasChildNodes())
-    languageMenuList.removeChild(languageMenuList.firstChild);
+    languageMenuList.lastChild.remove();
 
   for (let i = 0; i < count; i++)
   {
@@ -1869,7 +1870,7 @@ function ClearIdentityListPopup(popup)
 {
   if (popup)
     while (popup.hasChildNodes())
-      popup.removeChild(popup.lastChild);
+      popup.lastChild.remove();
 }
 
 function FillIdentityList(menulist)
@@ -1979,9 +1980,11 @@ function ComposeCanClose()
     // call window.focus, since we need to pop up a dialog
     // and therefore need to be visible (to prevent user confusion)
     window.focus();
+    let draftFolderURI = gCurrentIdentity.draftFolder;
+    let draftFolderName = MailUtils.getFolderForURI(draftFolderURI).prettyName;
     switch (Services.prompt.confirmEx(window,
               sComposeMsgsBundle.getString("saveDlogTitle"),
-              sComposeMsgsBundle.getString("saveDlogMessage"),
+              sComposeMsgsBundle.getFormattedString("saveDlogMessages",[draftFolderName]),
               (Services.prompt.BUTTON_TITLE_SAVE * Services.prompt.BUTTON_POS_0) +
               (Services.prompt.BUTTON_TITLE_CANCEL * Services.prompt.BUTTON_POS_1) +
               (Services.prompt.BUTTON_TITLE_DONT_SAVE * Services.prompt.BUTTON_POS_2),
@@ -2326,82 +2329,23 @@ function OpenSelectedAttachment()
 
 function DetermineHTMLAction(convertible)
 {
-    var obj;
-    if (! gMsgCompose.composeHTML)
-    {
-        try {
-          obj = new Object;
-          gMsgCompose.checkAndPopulateRecipients(true, false, obj);
-        } catch(ex) {
-          dump("gMsgCompose.checkAndPopulateRecipients failed: " + ex + "\n");
-        }
-        return nsIMsgCompSendFormat.PlainText;
-    }
+  try {
+    gMsgCompose.expandMailingLists();
+  } catch(ex) {
+    dump("gMsgCompose.expandMailingLists failed: " + ex + "\n");
+  }
 
-    if (gSendFormat == nsIMsgCompSendFormat.AskUser)
-    {
-        //Well, before we ask, see if we can figure out what to do for ourselves
+  if (!gMsgCompose.composeHTML)
+  {
+    return nsIMsgCompSendFormat.PlainText;
+  }
 
-        var noHtmlRecipients;
-        var noHtmlnewsgroups;
-        var preferFormat;
+  if (gSendFormat == nsIMsgCompSendFormat.AskUser)
+  {
+    return gMsgCompose.determineHTMLAction(convertible);
+  }
 
-        //Check the address book for the HTML property for each recipient
-        try {
-          obj = new Object;
-          preferFormat = gMsgCompose.checkAndPopulateRecipients(true, true, obj);
-          noHtmlRecipients = obj.value;
-        } catch(ex) {
-          dump("gMsgCompose.checkAndPopulateRecipients failed: " + ex + "\n");
-          var msgCompFields = gMsgCompose.compFields;
-          noHtmlRecipients = msgCompFields.to + "," + msgCompFields.cc + "," + msgCompFields.bcc;
-          preferFormat = nsIAbPreferMailFormat.unknown;
-        }
-
-        //Check newsgroups now...
-        noHtmlnewsgroups = gMsgCompose.compFields.newsgroups;
-
-        if (noHtmlRecipients != "" || noHtmlnewsgroups != "")
-        {
-            if (convertible == nsIMsgCompConvertible.Plain)
-              return nsIMsgCompSendFormat.PlainText;
-
-            if (noHtmlnewsgroups == "")
-            {
-                switch (preferFormat)
-                {
-                  case nsIAbPreferMailFormat.plaintext :
-                    return nsIMsgCompSendFormat.PlainText;
-
-                  default :
-                    //See if a preference has been set to tell us what to do. Note that we do not honor that
-                    //preference for newsgroups. Only for e-mail addresses.
-                    var action = getPref("mail.default_html_action");
-                    switch (action)
-                    {
-                        case nsIMsgCompSendFormat.PlainText    :
-                        case nsIMsgCompSendFormat.HTML         :
-                        case nsIMsgCompSendFormat.Both         :
-                            return action;
-                    }
-                }
-            }
-            return nsIMsgCompSendFormat.AskUser;
-        }
-        else
-            return nsIMsgCompSendFormat.HTML;
-    }
-    else
-    {
-      try {
-        obj = new Object;
-        gMsgCompose.checkAndPopulateRecipients(true, false, obj);
-      } catch(ex) {
-        dump("gMsgCompose.checkAndPopulateRecipients failed: " + ex + "\n");
-      }
-    }
-
-    return gSendFormat;
+  return gSendFormat;
 }
 
 function DetermineConvertibility()
